@@ -14,7 +14,9 @@ setClass("Fa", representation(call="language",
 			      loadings="matrix",
 			      communality="Uvector",
 			      uniquenesses="vector",
+				  covariance="matrix",
 			      correlation="matrix",
+				  usedMatrix="matrix",
 			      criteria="Unumeric",
 			      factors="numeric",
 			      dof="Unumeric",
