@@ -4,7 +4,7 @@ FaCov <- function (x, ...)
 
 # setMethod("getQuan", "FaCov", function(obj) obj@n.obs)
 
-FaCov.formula <- function (formula, data = NULL, factors = 2, method = "mle", scoresMethod = "none", ...)
+FaCov.formula <- function (formula, data = NULL, factors = 2, cor = FALSE, method = "mle", scoresMethod = "none", ...)
 ## formula and data arguments are used to construct x
 ## method = c("mle", "pca", "pfa"), scoresMethod = c("none", "regression", "Bartlett")
 ## subset, na.action ignored
@@ -19,17 +19,19 @@ FaCov.formula <- function (formula, data = NULL, factors = 2, method = "mle", sc
     if (!is.null(factors)) mf$factors <- NULL
     if (!is.null(method)) mf$method <- NULL
     if (!is.null(scoresMethod)) mf$scoresMethod <- NULL
+	if (!is.null(cor)) mf$cor <- NULL
     ## so that mf has ONLY 3 elements to be compatible with the following code "mf <- eval.parent(mf)"
 
     mf[[1]] <- as.name("model.frame")
     ## mf = model.frame(formula = ~., data = list(x1 = c(1531125205, 106581996.9,...
-    ## mf[[1]]=model.frame of class "name"; mf[[2]]=~. of class "formula"; mf[[3]]=data of class "data.frame"
+    ## mf[[1]] = model.frame of class "name"; mf[[2]] = ~. of class "formula"; mf[[3]] = data of class "data.frame"
 
     mf <- eval.parent(mf)
     ## this is not a 'standard' model-fitting function,
     ## so no need to consider contrasts or levels
-    if (rrcov:::.check_vars_numeric(mf)) # use rrcov:::.check_vars_numeric to see the function
-        stop("Fa applies only to numerical variables")
+
+	## if (rrcov:::.check_vars_numeric(mf)) # Unexported object imported by a ::: call
+    ##     stop("Fa applies only to numerical variables")
 
     na.act <- attr(mf, "na.action")
     mt <- attr(mf, "terms")
@@ -44,8 +46,8 @@ FaCov.formula <- function (formula, data = NULL, factors = 2, method = "mle", sc
     res
 }
 
-FaCov.default <- function(x, factors=2, cov.control = CovControlMcd(), method = c("mle", "pca", "pfa"), scoresMethod = c("none", "regression", "Bartlett"), ...) 
-# na.action = na.fail, trace=FALSE
+FaCov.default <- function(x, factors = 2, cor = FALSE, cov.control = CovControlMcd(), method = c("mle", "pca", "pfa"), scoresMethod = c("none", "regression", "Bartlett"), ...) 
+# na.action = na.fail, trace = FALSE
 {
 
     cl <- match.call()
@@ -60,8 +62,8 @@ FaCov.default <- function(x, factors=2, cov.control = CovControlMcd(), method =
     if(n < p)
         stop("'FaCov' can only be used with more units than variables")
     
-    if (missing(method)) method="mle"
-    if (missing(scoresMethod)) scoresMethod="none"
+    if (missing(method)) method = "mle"
+    if (missing(scoresMethod)) scoresMethod = "none"
 
     ##
     ## verify and set the input parameters: k and kmax
@@ -70,35 +72,36 @@ FaCov.default <- function(x, factors=2, cov.control = CovControlMcd(), method =
     # if((k <- floor(k)) < 0)
     #     k <- 0
     # else if(k > kmax) {
-    #     warning(paste("The number of principal components k = ", k, " is larger than kmax = ", kmax, "; k is set to ", kmax,".", sep=""))
+    #     warning(paste("The number of principal components k = ", k, " is larger than kmax = ", kmax, "; k is set to ", kmax,".", sep = ""))
     #     k <- kmax
     # }
-    # if(k != 0)
+    # if(k ! =  0)
     #     k <- min(k, p)
     # else {
     #     k <- min(kmax, p)
     #     if(trace)
-    #         cat("The number of principal components is defined by the algorithm. It is set to ", k,".\n", sep="")
+    #         cat("The number of principal components is defined by the algorithm. It is set to ", k,".\n", sep = "")
     # }
 
 ######################################################################
 
     ## add the option for classic covariance estimates - if cov.control = NULL
     covx <- if(!is.null(cov.control)) restimate(cov.control, data) else Cov(data)
-    covmat <- list(cov=getCov(covx), center=getCenter(covx), n.obs=covx@n.obs)
+    covmat <- list(cov = getCov(covx), center = getCenter(covx), n.obs = covx@n.obs)
 
     out <- switch(method, 
-                  pca=factorScorePca(factors=factors, covmat=covmat), # "factorScorePca" does not have "cor" argument
-                  pfa=factorScorePfa(factors=factors, covmat=covmat), # "factorScorePfa" does not have "cor" argument
-                  mle=factanal(factors=factors, covmat=covmat)) # "factanal" does not have "cor" argument
+                  pca = factorScorePca(x = data, factors = factors, covmat = covmat, cor = cor, scoresMethod = scoresMethod), 
+                  pfa = factorScorePfa(x = data, factors = factors, covmat = covmat, cor = cor, scoresMethod = scoresMethod),
+                  mle = factanal(factors = factors, covmat = covmat)) # "factanal" does not have "cor" argument
+	
+	if(scoresMethod !=  "none" && method ==  "mle")
+	out <- computeScores(out, x = data, covmat = covmat, cor = cor, scoresMethod = scoresMethod) # "computeScores" is defined in "utils.R"
 
-    outComputeScores <- computeScores(out, newdata=scale(data, center=covmat$center, scale = F), scoresMethod=scoresMethod) # "computeScores" is defined in "utils.R"
-
-    # scores   <- predict(out, newdata=data)
+    # scores   <- predict(out, newdata = data)
     # center   <- getCenter(covx)
     # sdev     <- out$sdev
-    # scores   <- scores[, 1:k, drop=FALSE]
-    # loadings <- out$loadings # [, 1:k, drop=FALSE]
+    # scores   <- scores[, 1:k, drop = FALSE]
+    # loadings <- out$loadings # [, 1:k, drop = FALSE]
     # eigenvalues  <- (sdev^2)[1:k]
 
 ######################################################################
@@ -112,27 +115,29 @@ FaCov.default <- function(x, factors=2, cov.control = CovControlMcd(), method =
 
     ## fix up call to refer to the generic, but leave arg name as `formula'
     cl[[1]] <- as.name("FaCov")
-    res <- new("FaCov", call=cl,
-			converged=out$converged,
-			loadings=out$loadings[], # class(out$loadings)=="loadings", class(out$loadings[])=="matrix"
-			communality=out$communality,
-			uniquenesses=out$uniquenesses,
-			correlation=out$correlation,
-			criteria=out$criteria,
-			factors=out$factors,
-			dof=out$dof,
-			method=out$method,
-			scores=outComputeScores$F,
-			scoresMethod=scoresMethod,
-			scoringCoef=outComputeScores$scoringCoef,
-			meanF=outComputeScores$meanF,
-			corF=outComputeScores$corF,
-			STATISTIC=out$STATISTIC,
-			PVAL=out$PVAL,
-			n.obs=n,
-			center=getCenter(covx),
-			eigenvalues=eigen(out$correlation)$values,
-			cov.control=cov.control)
+    res <- new("FaCov", call = cl,
+			converged = out$converged,
+			loadings = out$loadings[], # class(out$loadings) == "loadings", class(out$loadings[]) == "matrix"
+			communality = out$communality,
+			uniquenesses = out$uniquenesses,
+			covariance = out$covariance,
+			correlation = out$correlation,
+			usedMatrix = out$usedMatrix,
+			criteria = out$criteria,
+			factors = out$factors,
+			dof = out$dof,
+			method = out$method,
+			scores = out$scores,
+			scoresMethod = scoresMethod,
+			scoringCoef = out$scoringCoef,
+			meanF = out$meanF,
+			corF = out$corF,
+			STATISTIC = out$STATISTIC,
+			PVAL = out$PVAL,
+			n.obs = n,
+			center = getCenter(covx),
+			eigenvalues = out$eigenvalues,
+			cov.control = cov.control)
 
     ## Compute distances and flags
     # res <- rrcov:::.distances(x, p, res) # see the function ".distances.R"
